/* $Id: parsehtml.h,v 1.7 2002/12/18 10:10:48 aa5779 Exp $ */
/*
 	Copyright (C) 2001, 2002 Artem V. Andreev (artem@AA5779.spb.edu)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef PARSEHTML_H
#define PARSEHTML_H

#include "lists.h"
#include "dstring.h"

/*** \group[Declarations in \href[parsehtml.h]] */

/**** \struct[html_param]
represents a single attribute of a chunk
\field[name] name of the attribute
\field[val] value of the attribute; may be NULL
*/
DEFLIST(html_param,
    dstring *name;
    dstring *val;
);

/*** \struct
represents a markup element (a tag or a character)
\field[is_dynamic] = 1 if the chunk was dynamically allocated; 0 othewise
\field[type] is one of \see[enums][html_chunk_types]
\field[x.name] the name of the chunk if its type is tag
\field[x.ch] the character value of the chunk if its type is character
\field[params] the attribute list of the chunk if its type is tag; unspecified otherwise
*/
typedef struct html_chunk
{
	int is_dynamic;
    int type;
    union
    {
		dstring *name;
		int ch;
    } x;
    html_param *params;
} html_chunk;

/*** \enum
Possible chunk types
\field[hct_none] empty chunk
\field[hct_char] character chunk
\field[hct_tag] tag chunk
*/
enum html_chunk_types { hct_none, hct_char, hct_tag};

html_chunk *alloc_html_chunk(void);
html_chunk *copy_html_chunk(html_chunk *hc);

/*** \macro[HTML_CHUNK_TYPE] yields the type of a chunk */
#define HTML_CHUNK_TYPE(c) ((c)->type)

char *get_html_param(html_chunk *chunk, char *name);
void set_html_param(html_chunk *chunk, dstring *name, dstring *val);
void remove_html_param(html_chunk *chunk, dstring *name);
void clear_html_params(html_chunk *chunk);
void set_tag_name(html_chunk *chunk, dstring *newname);
char *get_tag_name(html_chunk *chunk);

#define HTML_NO_CHAR (-1)

int get_html_char(html_chunk *chunk);
void set_html_char(html_chunk *chunk, int ch);

void destroy_html_param(html_param *parm);

void release_html_chunk(html_chunk *chunk);
void reset_html_chunk(html_chunk *chunk);

#endif
/*** \endgroup */
