/* $Id: pflow.c,v 1.10 2002/11/05 10:20:39 aa5779 Exp $ */
/*
	Copyright (C) 2001, 2002 Artem V. Andreev (artem@AA5779.spb.edu)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>
#include <stdio.h>
#undef getc
#undef putc
#undef puts
#include "flow.h"
#define PFLOW_IMP
#include "pflow.h"
#include "util.h"

flow_class pipe_flow_class = {"pipe", pf_open, pf_close, pf_getc, pf_ungetc, 
							   pf_putc, pf_puts,
							   pf_getpos, pf_setpos, pf_rewind, pf_flowctl
							 };

flow_handle_t pf_open(char *name, char *opts, int writeable)
{
#ifdef HAVE_POPEN
	FILE *pf = popen(name, writeable ? "w" : "r");
	if(!pf)
		recover_error("can't open pipe flow '%s': %s", name, strerror(errno));
	return pf;
#else
	recover_error("pipes are not supported");
#endif
}

void pf_close(flow_handle_t fh)
{
#ifdef HAVE_POPEN
	if(pclose(fh) < 0)
		recover_error("pipe flow close error: %s", strerror(errno));
#endif
}

int pf_getc(flow_handle_t fh)
{
	int ch = fgetc(fh);
	return ch == EOF ? FLOW_EOF : ch;
}

int pf_ungetc(flow_handle_t fh, int ch)
{
	ungetc(ch, fh);
	return 0;
}

int pf_putc(flow_handle_t fh, int ch)
{
	fputc(ch, fh);
	return 0;
}

int pf_puts(flow_handle_t fh, char *str)
{
	fputs(str, fh);
	return 0;
}
 
flow_pos_t pf_getpos(flow_handle_t fh)
{
	return FLOW_ERROR;
}

int pf_setpos(flow_handle_t fh, flow_pos_t fp)
{
	return FLOW_ERROR;
}

int pf_rewind(flow_handle_t fh)
{
	return FLOW_ERROR;
}

char *pf_flowctl(flow_handle_t fh, char *ctl)
{
	FILE *pf = fh;
	if(!strcmp(ctl, "isatty"))
		return "";
	if(!strcmp(ctl, "flush"))
	{
		fflush(pf);
		return "";
	}
	else if(!strcmp(ctl, "unbuffer"))
	{
		setvbuf(pf, NULL, _IONBF, BUFSIZ);
		return "";
	}
	else if(!strcmp(ctl, "lbuffer"))
	{
		setvbuf(pf, NULL, _IOLBF, BUFSIZ);
		return "";
	}
	else if(!strcmp(ctl, "buffer"))
	{
		setvbuf(pf, NULL, _IOFBF, BUFSIZ);
		return "";
	}
	else if(!strcmp(ctl, "lock") || !strcmp(ctl, "unlock") || !strcmp(ctl, "lock?") ||
		!strcmp(ctl, "loaded?"))
		return "";
	return NULL;
}
