/* $Id: sla.c,v 1.13 2002/12/09 10:38:14 aa5779 Exp $ */
/*
	Copyright (C) 2001, 2002 Artem V. Andreev (artem@AA5779.spb.edu)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
/// \author[Artem V. Andreev]
/// \copyright[2001, 2002 Artem V. Andreev. See \ref[sl(1)] for details]
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#include <errno.h>
#include <limits.h>
#include "lists.h"
#include "util.h"
#include "shared.h"
#include "automat.h"
/// \section[NAME]
// \em[sla] -- an interpreter for push-down transducers

/// \section[SYNOPSIS]
// sla \em[autoseq] <infile >outfile

/// \section[DESCRIPTION]
// \em[sla] is a simple tool to apply sequences of \em[push-down transducers (PDTs)] to
// text files. It works as a filter only. 
// \em[sla] should normally come along with Tensile interpreter.
//
// The PDT stuff described below is also applicable to Tensile (see \ref[sl(1)]).


/// \include[automat.c]

shared_object_class *get_so_class(char *clsname)
{
	recover_error("internal error");
}

#define CHUNK_SIZE (128 * 1024)
static char *chunk_buffer, *cur_ptr;
static int read_size;

int my_getchar()
{
	if(!read_size)
	{
		read_size = fread(chunk_buffer, 1, CHUNK_SIZE, stdin);
		if(read_size <= 0)
			return EOF;
		cur_ptr = chunk_buffer;
	}
	read_size--;
	return *cur_ptr++;
}

int main(int argc, char *argv[])
{
	autoseq *au;
	int ch;
	int res = 0;
	int out;
	
	if(argc < 2)
	{
		fputs("Usage: sla autoseq-name\n", stderr);
		exit(1);
	}
	cur_ptr = chunk_buffer = xcalloc(CHUNK_SIZE, sizeof *chunk_buffer);
	setenv(AU_PATH_ENV, AU_LOCATION, 0);
	au = parse_autoseq(argv[1]);
	autoseq_init(au);
	while((ch = my_getchar()) != EOF)
	{
		for(;;)
		{
			res = autoseq_step(au, &ch, &out);
			if(out > UCHAR_MAX)
				fatal_error("The Unicode character 'U+%4.4x' doesn't fit 8 bit", out);
			if(out >= 0)
				putchar(out);
			if(ch < 0 && need_feeding(au))
				break;
		}
		if(res == ar_rejected)
			fatal_error("input sequence rejected");
	}
	ch = 0;
	while(ch >= 0 || !need_feeding(au))
	{
		res = autoseq_step(au, &ch, &out);
		if(out > UCHAR_MAX)
			fatal_error("The Unicode character 'U+%4.4x' doesn't fit 8 bit", out);
		if(out > 0)
			putchar(out);
	}
	if(res == ar_rejected)
		fatal_error("input sequence rejected");
	return EXIT_SUCCESS;
}

/// \trailer
